@@ -15,9 +15,12 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with woob. If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import annotations
+
 import datetime
 import re
-import unicodedata
+import pycountry
+from typing import Any
 from collections.abc import Iterator
 from decimal import Decimal, InvalidOperation
 from itertools import islice
@@ -27,11 +30,11 @@ from urllib.parse import parse_qs, urlparse
 from dateutil.parser import parse as parse_date
 from dateutil.tz import gettz
 from lxml.etree import ElementBase as LXMLElement
-import unidecode
 
 from woob.browser.url import URL
 from woob.capabilities.base import Currency as BaseCurrency
 from woob.capabilities.base import empty
+from woob.tools.misc import clean_text
 
 from .base import _NO_DEFAULT, Filter, FilterError, ItemNotFound, _Filter, debug
 
@@ -42,7 +45,7 @@ __all__ = [
     'CleanDecimal', 'Slugify', 'Type', 'Field', 'Regexp', 'Map', 'MapIn',
     'DateTime', 'FromTimestamp', 'Date', 'DateGuesser', 'Time', 'Duration',
     'MultiFilter', 'CombineDate', 'Format', 'BrowserURL', 'Join', 'MultiJoin',
-    'Eval', 'QueryValue', 'Coalesce',
+    'Eval', 'QueryValue', 'Coalesce', 'CountryCode',
 ]
 
 
@@ -284,19 +287,12 @@ class CleanText(Filter):
         elif not isinstance(txt, str):
             txt = ' '.join(txt.itertext())
 
-        if newlines:
-            txt = re.compile(r'\s+', flags=re.UNICODE).sub(' ', txt)  # 'foo bar '
-        else:
-            # normalize newlines and clean what is inside
-            txt = '\n'.join([cls.clean(l) for l in txt.splitlines()])
-
-        txt = txt.strip()
-        # normalize to a standard Unicode form
-        if normalize:
-            txt = unicodedata.normalize(normalize, txt)
-        if transliterate:
-            txt = unidecode.unidecode(txt)
-        return txt
+        return clean_text(
+            txt,
+            remove_newlines=newlines,
+            normalize=normalize,
+            transliterate=transliterate,
+        )
 
     @classmethod
     def remove(cls, txt, symbols):
@@ -347,6 +343,35 @@ class Currency(CleanText):
         return BaseCurrency.get_currency(txt)
 
 
+class CountryCode(CleanText):
+    """
+    Filter to get the country ISO 3166-1 alpha-2 code from the country name
+    """
+    @debug()
+    def filter(self, txt: str) -> Any:
+        """Get the country code from the name of the country
+
+        :param txt: Country name
+        :type txt: str
+        :raises FormatError: if the Country name is not found
+
+        >>> CountryCode().filter('france')
+        'fr'
+        >>> CountryCode(default= 'll').filter('Greez')
+        'll'
+        """
+        txt = super().filter(txt)
+        if empty(txt):
+            return self.default_or_raise(FormatError("Unable to parse %r" % txt))
+        try:
+            alpha_2_code = pycountry.countries.lookup(txt).alpha_2.lower()
+        except LookupError:
+            # Country not found
+            return self.default_or_raise(FormatError('Country not recognized: %r' % txt))
+
+        return alpha_2_code
+
+
 class NumberFormatError(FormatError, InvalidOperation):
     pass
 
@@ -363,7 +388,7 @@ class CleanDecimal(CleanText):
     If `replace_dots` is a tuple, the first element will be used as the thousands separator,
     and the second as the decimal separator.
 
-    See http://en.wikipedia.org/wiki/Thousands_separator#Examples_of_use
+    See https://en.wikipedia.org/wiki/Thousands_separator#Examples_of_use
 
     For example, for the UK style (as in 1,234,567.89):
 
@@ -383,6 +408,7 @@ class CleanDecimal(CleanText):
             thousands_sep, decimal_sep = self.replace_dots
             self.matching = re.compile(r'([+-]?)\s*(\d[\d%s%s]*|%s\d+)' % tuple(map(re.escape, (thousands_sep, decimal_sep, decimal_sep))))
             self.thousand_check = re.compile(r'^[+-]?\d{1,3}(%s\d{3})*(%s\d*)?$' % tuple(map(re.escape, (thousands_sep, decimal_sep))))
+            self.is_scientific_notation = re.compile(r'([+-]?)(\d+(?:[.,]\d*)?[eE][+-]?\d+)')
 
     @debug()
     def filter(self, text):
@@ -412,7 +438,9 @@ class CleanDecimal(CleanText):
             if not matches:
                 return self.default_or_raise(NumberFormatError('There is no number to parse'))
             elif len(matches) > 1:
-                return self.default_or_raise(NumberFormatError('There should be exactly one number to parse'))
+                matches = self.is_scientific_notation.findall(text)
+                if not matches:
+                    return self.default_or_raise(NumberFormatError('There should be exactly one number to parse'))
 
             text = '%s%s' % (matches[0][0], matches[0][1].strip())
 
@@ -794,7 +822,7 @@ class DateGuesser(Filter):
 class Time(Filter):
     """Parse time."""
 
-    klass = datetime.time
+    klass: type = datetime.time
     _regexp = re.compile(r'(?P<hh>\d+)[:h]?(?P<mm>\d+)([:m](?P<ss>\d+))?')
     kwargs = {'hour': 'hh', 'minute': 'mm', 'second': 'ss'}
 
@@ -816,7 +844,7 @@ class Time(Filter):
 class Duration(Time):
     """Parse a duration as timedelta."""
 
-    klass = datetime.timedelta
+    klass: type = datetime.timedelta
     _regexp = re.compile(r'((?P<hh>\d+)[:;])?(?P<mm>\d+)[;:](?P<ss>\d+)')
     kwargs = {'hours': 'hh', 'minutes': 'mm', 'seconds': 'ss'}
 
@@ -1014,7 +1042,7 @@ class QueryValue(Filter):
     >>> from lxml.html import etree
     >>> from .html import Link
     >>> f = QueryValue(Link('//a'), 'id')
-    >>> f(etree.fromstring('<html><body><a href="http://example.org/view?id=1234"></a></body></html>')) == u'1234'
+    >>> f(etree.fromstring('<html><body><a href="https://example.org/view?id=1234"></a></body></html>')) == u'1234'
     True
     """
     def __init__(self, selector, key, default=_NO_DEFAULT):
