@@ -15,16 +15,17 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with woob. If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import annotations
 
 from binascii import crc32
 import re
-from typing import Optional, Iterator, List
+from typing import Iterable, List
 
 from woob.capabilities.account import CapCredentialsCheck
 from woob.capabilities.base import (
     BaseObject, Capability, Field, StringField, DecimalField, IntField,
-    BoolField, UserError, Currency, NotAvailable, EnumField, Enum,
-    empty, find_object,
+    BoolField, UserError, Currency, NotAvailable, EnumField, Enum, empty,
+    find_object, NotLoaded, NotLoadedType, NotAvailableType
 )
 from woob.capabilities.date import DateField
 from woob.capabilities.collection import CapCollection
@@ -34,11 +35,20 @@ from woob.exceptions import BrowserIncorrectPassword
 __all__ = [
     'CapBank', 'BaseAccount', 'Account', 'Loan', 'Transaction', 'AccountNotFound',
     'AccountType', 'AccountOwnership', 'Balance', 'AccountSchemeName', 'TransactionCounterparty',
-    'AccountOwnerProfile', 'IdentityParty', 'AccountParty', 'AccountIdentification',
-    'PartyRole', 'CapAccountCheck',
+    'PartyIdentity', 'AccountParty', 'AccountIdentification', 'PartyRole', 'CapAccountCheck',
+    'NoAccountsException', 'BalanceType', 'BankTransactionCode',
 ]
 
 
+class NoAccountsException(Exception):
+    """
+    Raised by :meth:`CapBank.iter_accounts()` if we are sure there is no accounts.
+
+    Sometimes we can't parse find accounts on websites but that's a scraping
+    error. To attest we know that's a real case, this exception is raised.
+    """
+
+
 class ObjectNotFound(UserError):
     pass
 
@@ -48,7 +58,7 @@ class AccountNotFound(ObjectNotFound):
     Raised when an account is not found.
     """
 
-    def __init__(self, msg: Optional[str] = 'Account not found'):
+    def __init__(self, msg: str = 'Account not found'):
         super().__init__(msg)
 
 
@@ -61,7 +71,13 @@ class BaseAccount(BaseObject, Currency):
     currency =       StringField('Currency', default=None)
     bank_name =      StringField('Bank Name', mandatory=False)
 
-    def __init__(self, id: Optional[str] = '0', url: Optional[str] = None):
+    # TODO add iban field here
+
+    def __init__(
+        self,
+        id: str = '0',
+        url: str | NotLoadedType | NotAvailableType = NotLoaded
+    ):
         super().__init__(id, url)
 
     @property
@@ -69,7 +85,7 @@ class BaseAccount(BaseObject, Currency):
         return Currency.currency2txt(self.currency)
 
     @property
-    def ban(self) -> str:
+    def ban(self) -> str | NotAvailableType:
         """ Bank Account Number part of IBAN"""
         if not self.iban:
             return NotAvailable
@@ -122,6 +138,20 @@ class AccountType(Enum):
     "Real estate investment such as SCPI, OPCI, SCI"
     CROWDLENDING = 22
     "Crowdlending accounts"
+    LDDS = 23
+    "LDD/LDDS Livret de développement durable et solidaire"
+    PEL = 24
+    "Plan épargne logement"
+    CSL = 25
+    "Compte sur Livret"
+    CEL = 26
+    "Compte épargne logement"
+    CAT = 27
+    "Compte à terme"
+    LIVRET_A = 28
+    "Livret A"
+    LIVRET_B = 29
+    "Livret B"
 
 
 class AccountOwnerType:
@@ -161,6 +191,30 @@ class AccountSchemeName(Enum):
     CPAN = 'cpan'
     """Card PAN (masked or plain)"""
 
+    TPAN = 'tpan'
+    """Tokenized card PAN issued by a Token Service Provider to obfuscate the real PAN"""
+
+    MPAN = 'mpan'
+    """Card PAN where some digits were replaced for security reason"""
+
+    BANK_PARTY_IDENTIFICATION = 'bank_party_identification'
+    """
+    BankPartyIdentification - Unique and unambiguous assignment made by a specific bank or
+    similar financial institution to identify a relationship between the bank and its client.
+
+    Its definition can be found at page 13 of https://www.stet.eu/assets/files/PSD2/1-6-3/api-dsp2-stet-v1.6.3.1-part-2-functional-model.pdf
+    """
+
+
+class AccountManagementType(Enum):
+    UNKNOWN = 'unknown'
+    CAPITALIZATION = 'capitalization'
+    FIXED_FUNDS = 'fixed_funds'
+    PROFILED = 'profiled'
+    DISCRETIONARY = 'discretionary'
+    DELEGATED = 'delegated'
+    UNIT_LINKED = 'unit_linked'
+
 
 class TransactionCounterparty(BaseObject):
     label = StringField('Name of the other stakeholder (Creditor or debtor)', default=None)
@@ -172,19 +226,6 @@ class TransactionCounterparty(BaseObject):
         return f'<label={self.label} debtor={self.debtor} account_scheme_name={self.account_scheme_name} account_identification={self.account_identification}>'
 
 
-class AccountOwnerProfile(TransactionCounterparty):
-    """
-    AccountOwnerProfile contains the information related to the account and its owner:
-    - account_scheme_name: scheme type of the account (IBAN or sort code + account number)
-    - account_identification: value that identifies the account and is related to the scheme type
-    - holder_name: full name of the account owner
-    """
-    holder_name = StringField('Name of the account holder', default=None)
-
-    def __repr__(self):
-        return f'<AccountOwnerProfile account_scheme_name={self.account_scheme_name} account_identification={self.account_identification} holder_name={self.holder_name}>'
-
-
 class PartyRole(Enum):
     UNKNOWN = 'unknown'
     HOLDER = 'holder'
@@ -192,7 +233,8 @@ class PartyRole(Enum):
     ATTORNEY = 'attorney'
     CUSTODIAN_FOR_MINOR = 'custodian_for_minor'
     LEGAL_GUARDIAN = 'legal_guardian'
-    NOMINEE_BENEFICIARY = 'nominee_beneficiary'
+    NOMINEE = 'nominee'
+    BENEFICIARY = 'beneficiary'
     SUCCESSOR_ON_DEATH = 'successor_on_death'
     TRUSTEE = 'trustee'
 
@@ -210,7 +252,7 @@ class AccountIdentification(BaseObject):
         return f'<AccountIdentification scheme_name={self.scheme_name} identification={self.identification}>'
 
 
-class IdentityParty(BaseObject):
+class PartyIdentity(BaseObject):
     """
     Defines the identity of a party:
     - full_name: Full name of the party
@@ -223,7 +265,8 @@ class IdentityParty(BaseObject):
     ROLE_ATTORNEY = PartyRole.ATTORNEY
     ROLE_CUSTODIAN_FOR_MINOR = PartyRole.CUSTODIAN_FOR_MINOR
     ROLE_LEGAL_GUARDIAN = PartyRole.LEGAL_GUARDIAN
-    ROLE_NOMINEE_BENEFICIARY = PartyRole.NOMINEE_BENEFICIARY
+    ROLE_NOMINEE = PartyRole.NOMINEE
+    ROLE_BENEFICIARY = PartyRole.BENEFICIARY
     ROLE_SUCCESSOR_ON_DEATH = PartyRole.SUCCESSOR_ON_DEATH
     ROLE_TRUSTEE = PartyRole.TRUSTEE
 
@@ -232,20 +275,20 @@ class IdentityParty(BaseObject):
     role = EnumField('Role of the party.', PartyRole, default=ROLE_UNKNOWN)
 
     def __repr__(self):
-        return f'<IdentityParty full_name={self.full_name} role={self.role} is_user={self.is_user}>'
+        return f'<PartyIdentity full_name={self.full_name} role={self.role} is_user={self.is_user}>'
 
 
 class AccountParty(BaseObject):
     """
     Defines all the information related to an account party:
-    - identities: list of IdentityParty elements
-    - identifications : list of AccountIdentification elements
+    - party_identities: list of PartyIdentity elements
+    - account_identifications : list of AccountIdentification elements
     """
-    identities = Field('Identities of the account party', list, default=[])
-    identifications = Field('Identification information of the account', list, default=[])
+    party_identities = Field('Identities of the account party', list, default=[])
+    account_identifications = Field('Identification information of the account', list, default=[])
 
     def __repr__(self):
-        return f'<AccountParty identities={self.identities} identifications={self.identifications}>'
+        return f'<AccountParty party_identities={self.party_identities} account_identifications={self.account_identifications}>'
 
 
 class Account(BaseAccount):
@@ -275,6 +318,13 @@ class Account(BaseAccount):
     TYPE_PER              = AccountType.PER
     TYPE_REAL_ESTATE      = AccountType.REAL_ESTATE
     TYPE_CROWDLENDING     = AccountType.CROWDLENDING
+    TYPE_LDDS             = AccountType.LDDS
+    TYPE_PEL              = AccountType.PEL
+    TYPE_CSL              = AccountType.CSL
+    TYPE_CEL              = AccountType.CEL
+    TYPE_CAT              = AccountType.CAT
+    TYPE_LIVRET_A         = AccountType.LIVRET_A
+    TYPE_LIVRET_B         = AccountType.LIVRET_B
 
     type =      EnumField('Type of account', AccountType, default=TYPE_UNKNOWN)
     owner_type = StringField('Usage of account')  # cf AccountOwnerType class
@@ -293,6 +343,7 @@ class Account(BaseAccount):
     # Wealth accounts (market, life insurance...)
     valuation_diff = DecimalField('+/- values total')
     valuation_diff_ratio = DecimalField('+/- values ratio')
+    management_type = EnumField('Management type of account', AccountManagementType, default=None)
 
     # Employee savings (PERP, PERCO, Article 83...)
     company_name = StringField('Name of the company of the stock - only for employee savings')
@@ -308,9 +359,7 @@ class Account(BaseAccount):
 
     all_balances = Field('List of balances', list, default=[])
 
-    profile = Field('Profile associated to the account owner', AccountOwnerProfile)
-
-    party = Field('Party associated to the account', AccountParty, default=None, mandatory=False)
+    party = Field('Party associated to the account', AccountParty, default=None)
 
     def __repr__(self):
         return "<%s id=%r label=%r>" % (type(self).__name__, self.id, self.label)
@@ -344,9 +393,20 @@ class Balance(BaseObject):
     last_update = DateField('Last time balance was updated')
     credit_included = BoolField('If factoring is included in balance', default=False)
     label = StringField('Bank name of the balance')
+    calculated = BoolField('If computation has been made on the balance', default=False)
 
     def __repr__(self):
-        return f'<{type(self).__name__} label={self.label} amount={self.amount} type={self.type} credit_included={self.credit_included}>'
+        # Ex: '< Balance: label="Solde en Valeur" amount=972.94 type=1 credit_included=False reference_date=2023-06-09 >'
+        return ' '.join((
+            '<',
+            f'{type(self).__name__}:',
+            f'label="{self.label}"',
+            f'amount={self.amount}',
+            f'type={self.type}',
+            f'credit_included={self.credit_included}',
+            f'reference_date={self.reference_date}',
+            '>'
+        ))
 
 
 class Loan(Account):
@@ -367,6 +427,8 @@ class Loan(Account):
 
     subscription_date = DateField('Date of subscription of the loan')
     maturity_date = DateField('Estimated end date of the loan')
+    start_repayment_date = DateField('Date of start repayment of the loan')
+    deferred = BoolField('If loan is deferred')
     duration = IntField('Duration of the loan given in months')
     rate = DecimalField('Monthly rate of the loan')
 
@@ -395,6 +457,21 @@ class TransactionType(Enum):
     CARD_SUMMARY  = 11
     DEFERRED_CARD = 12
     INSTANT       = 13
+    MARKET_ORDER  = 14
+    MARKET_FEE    = 15
+    ARBITRAGE     = 16
+    PROFIT        = 17
+
+
+class BankTransactionCode(BaseObject):
+    """
+    Object dedicating to bank transaction codes
+    It follows the ISO20022 standards.
+    See https://www.iso20022.org/catalogue-messages/additional-content-messages/external-code-sets
+    """
+    domain = StringField('Domain of the transaction')
+    family = StringField('Family of the transaction')
+    sub_family = StringField('Sub-family of the transaction')
 
 
 class Transaction(BaseObject):
@@ -415,6 +492,10 @@ class Transaction(BaseObject):
     TYPE_CARD_SUMMARY  = TransactionType.CARD_SUMMARY
     TYPE_DEFERRED_CARD = TransactionType.DEFERRED_CARD
     TYPE_INSTANT       = TransactionType.INSTANT
+    TYPE_MARKET_ORDER  = TransactionType.MARKET_ORDER
+    TYPE_MARKET_FEE    = TransactionType.MARKET_FEE
+    TYPE_ARBITRAGE     = TransactionType.ARBITRAGE
+    TYPE_PROFIT        = TransactionType.PROFIT
 
     date =      DateField('Debit date on the bank statement')
     rdate =     DateField('Real date, when the payment has been made; usually extracted from the label or from credit card info')
@@ -425,6 +506,7 @@ class Transaction(BaseObject):
     category =  StringField('Category of the transaction')
     label =     StringField('Pretty label')
     amount = DecimalField('Net amount of the transaction, used to compute account balance')
+    coming = BoolField('True if the transaction is not yet booked')
 
     card =              StringField('Card number (if any)')
     commission =        DecimalField('Commission part on the transaction (in account currency)')
@@ -439,15 +521,18 @@ class Transaction(BaseObject):
     original_commission_currency = StringField('Currency of the original commission')
     original_gross_amount = DecimalField('Original gross amount (in another currency)')
 
+    attachments =       Field('List of files attached to the transaction', list)
     # Financial arbitrations
     investments =       Field('List of investments related to the transaction', list, default=[])
 
     counterparty = Field('Counterparty of transaction', TransactionCounterparty)
 
+    bank_transaction_code = Field('Bank transaction code of transaction', BankTransactionCode)
+
     def __repr__(self):
         return "<Transaction date=%r label=%r amount=%r>" % (self.date, self.label, self.amount)
 
-    def unique_id(self, seen: Optional[set] = None, account_id: Optional[str] = None) -> str:
+    def unique_id(self, seen: set | None = None, account_id: str | None = None) -> str:
         """
         Get an unique ID for the transaction based on date, amount and raw.
 
@@ -507,7 +592,7 @@ class CapBank(CapCollection, CapCredentialsCheck):
 
         return True
 
-    def iter_resources(self, objs: List[BaseObject], split_path: List[str]) -> Iterator[BaseObject]:
+    def iter_resources(self, objs: List[BaseObject], split_path: List[str]) -> Iterable[BaseObject]:
         """
         Iter resources.
 
@@ -523,9 +608,9 @@ class CapBank(CapCollection, CapCredentialsCheck):
         if Account in objs:
             self._restrict_level(split_path)
 
-            return self.iter_accounts()
+            yield from self.iter_accounts()
 
-    def iter_accounts(self) -> Iterator[Account]:
+    def iter_accounts(self) -> Iterable[Account]:
         """
         Iter accounts.
 
@@ -533,7 +618,7 @@ class CapBank(CapCollection, CapCredentialsCheck):
         """
         raise NotImplementedError()
 
-    def get_account(self, id: str) -> Account:
+    def get_account(self, id: str) -> Account | None:
         """
         Get an account from its ID.
 
@@ -544,26 +629,41 @@ class CapBank(CapCollection, CapCredentialsCheck):
         """
         return find_object(self.iter_accounts(), id=id, error=AccountNotFound)
 
-    def iter_history(self, account: Account) -> Iterator[Account]:
+    def iter_history(self, account: Account) -> Iterable[Transaction]:
         """
-        Iter history of transactions on a specific account.
+        Iter history of transactions of a specific account.
 
         :param account: account to get history
         :type account: :class:`Account`
         :rtype: iter[:class:`Transaction`]
         :raises: :class:`AccountNotFound`
         """
-        raise NotImplementedError()
+        return self.iter_transactions(account, with_coming=False)
 
-    def iter_coming(self, account: Account) -> Iterator[Transaction]:
+    def iter_coming(self, account: Account) -> Iterable[Transaction]:
         """
-        Iter coming transactions on a specific account.
+        Iter coming transactions of a specific account.
 
         :param account: account to get coming transactions
         :type account: :class:`Account`
         :rtype: iter[:class:`Transaction`]
         :raises: :class:`AccountNotFound`
         """
+        return self.iter_transactions(account, with_history=False)
+
+    def iter_transactions(
+        self, account: Account, *, with_history: bool = True, with_coming: bool = True
+    ) -> Iterable[Transaction]:
+        """
+        Iter all transactions (history and coming) of a specific account.
+
+        :param account: account to get transactions
+        :param with_history: if False, booked transactions will not be returned
+        :param with_coming: if False, coming transactions will not be returned
+        :type account: :class:`Account`
+        :rtype: iter[:class:`Transaction`]
+        :raises: :class:`AccountNotFound`
+        """
         raise NotImplementedError()
 
 
@@ -573,11 +673,11 @@ class CapAccountCheck(Capability):
 
     The expected structure is the following:
         - AccountParty object
-            * identities (list of IdentityParty elements):
+            * party_identities (list of PartyIdentity elements):
                 * full name
                 * role
                 * is_user
-            * identifications (list of type AccountIdentification elements):
+            * account_identifications (list of type AccountIdentification elements):
                 * scheme name
                 * identification
     """
