@@ -28,18 +28,21 @@ from compileall import compile_dir
 from contextlib import closing, contextmanager
 from datetime import datetime
 from io import BytesIO, StringIO
+from pathlib import Path
 from tempfile import NamedTemporaryFile, mkdtemp
 from urllib.request import getproxies
 from configparser import RawConfigParser, DEFAULTSECT
 import tarfile
 
 import packaging.version
+from packaging.specifiers import SpecifierSet
 
 from woob.browser.browsers import Browser
 from woob.browser.profiles import Woob as WoobProfile
 from woob.exceptions import BrowserHTTPError, BrowserHTTPNotFound, ModuleInstallError
 from woob.tools.log import getLogger
 from woob.tools.misc import get_backtrace, to_unicode, find_exe
+from woob.tools.packaging import parse_requirements
 
 from .modules import LoadedModule, _add_in_modules_path
 
@@ -75,6 +78,7 @@ class ModuleInfo:
         self.maintainer = ''
         self.license = ''
         self.icon = ''
+        self.woob_spec = None
 
     def load(self, items):
         self.version = int(items['version'])
@@ -84,6 +88,7 @@ class ModuleInfo:
         self.maintainer = to_unicode(items['maintainer'])
         self.license = to_unicode(items['license'])
         self.icon = items['icon'].strip() or None
+        self.woob_spec = SpecifierSet(items.get('woob_spec', ''))
 
     def has_caps(self, *caps):
         """Return True if module implements at least one of the caps."""
@@ -110,6 +115,7 @@ class ModuleInfo:
                 ('maintainer', self.maintainer),
                 ('license', self.license),
                 ('icon', self.icon or ''),
+                ('woob_spec', str(self.woob_spec)),
                )
 
 
@@ -303,9 +309,26 @@ class Repository:
 
         for name in sorted(os.listdir(path)):
             module_path = os.path.join(path, name)
-            if not os.path.isdir(module_path) or '.' in name or name == self.KEYDIR or not os.path.exists(os.path.join(module_path, '__init__.py')):
+
+            # Check for special cases.
+            if (
+                name.startswith('.')  # ".", "..", and private files and dirs
+                or name.startswith('_') or name.endswith('_')
+                or name == self.KEYDIR
+            ):
                 continue
 
+            # Check if the module is indeed a module.
+            if os.path.isdir(module_path):
+                if not os.path.exists(os.path.join(module_path, '__init__.py')):
+                    continue
+            else:
+                basename = os.path.basename(module_path).casefold()
+                if not basename.endswith('.py') or len(basename) < 3:
+                    continue
+
+                name = name[:-3]
+
             try:
                 pymodule = importlib.import_module(f'woob_modules.{name}')
                 module = LoadedModule(pymodule)
@@ -323,6 +346,14 @@ class Repository:
                 m.maintainer = module.maintainer
                 m.license = module.license
                 m.icon = module.icon or ''
+
+                module_path = Path(module.path)
+                if not os.path.isdir(module_path):
+                    module_path = module_path.parent
+
+                requirements = parse_requirements(module_path / 'requirements.txt')
+                m.woob_spec = requirements.get('woob', '')
+
                 self.modules[module.name] = m
 
         self.update = int(datetime.now().strftime('%Y%m%d%H%M'))
@@ -331,7 +362,7 @@ class Repository:
     @staticmethod
     def get_tree_mtime(path, include_root=False):
         mtime = 0
-        if include_root:
+        if include_root or not os.path.isdir(path):
             mtime = int(datetime.fromtimestamp(os.path.getmtime(path)).strftime('%Y%m%d%H%M'))
         for root, _, files in os.walk(path):
             for f in files:
@@ -498,7 +529,7 @@ def dependency_sort(deps_rules):
 
 
 DEFAULT_SOURCES_LIST = \
-"""# List of Woob repositories
+"""# List of woob repositories
 #
 # The entries below override the entries above (with
 # backends of the same name).
@@ -506,7 +537,7 @@ DEFAULT_SOURCES_LIST = \
 https://updates.woob.tech/%(version)s/main/
 
 # DEVELOPMENT
-# If you want to hack on Woob modules, you may add a
+# If you want to hack on woob modules, you may add a
 # reference to sources, for example:
 #file:///home/rom1/src/woob/woob_modules/
 """
@@ -824,6 +855,12 @@ class Repositories:
         :param progress: observer object
         :type progress: :class:`IProgress`
         """
+        if self.version not in module.woob_spec:
+            raise ModuleInstallError(
+                f"Module requires woob {module.woob_spec}, but you use woob {self.version}'.\n"
+                "Hint: use 'woob update' or install a newer version of woob"
+            )
+
         self.load_browser()
 
         if module.is_local():
@@ -859,8 +896,11 @@ class Repositories:
         if os.path.isdir(module_dir):
             shutil.rmtree(module_dir)
         progress.progress(0.7, 'Setting up module...')
+
+        # TODO: remove tar.extractall() when not needed to prevent from potential archive attacks
         with closing(tarfile.open('', 'r:gz', BytesIO(tardata))) as tar:
-            tar.extractall(self.modules_dir)
+            tar.extractall(self.modules_dir)  # nosec
+
         if not os.path.isdir(module_dir):
             raise ModuleInstallError(f'The archive for {module.name} looks invalid.')
         # Precompile
