@@ -15,20 +15,23 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with woob. If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import annotations
 
 import re
 import datetime
 from collections import OrderedDict
+from typing import TypeVar
 
 from .misc import to_unicode
 
 
 __all__ = ['ValuesDict', 'Value', 'ValueBackendPassword', 'ValueInt', 'ValueFloat', 'ValueBool']
 
+ValuesDictType = TypeVar('ValuesDictType', bound='ValuesDict')
+
 
 class ValuesDict(OrderedDict):
-    """
-    Ordered dictionarry which can take values in constructor.
+    """Ordered dictionary which can take values in constructor.
 
     >>> ValuesDict(Value('a', label='Test'), ValueInt('b', label='Test2'))
     """
@@ -38,6 +41,42 @@ class ValuesDict(OrderedDict):
         for v in values:
             self[v.id] = v
 
+    def with_values(self: ValuesDictType, *values: Value) -> ValuesDictType:
+        """Get a copy of the object, with new values.
+
+        :param values: The values to set.
+        :return: The new values dictionary.
+        """
+        existing_values = {key: value for key, value in self.items()}
+        existing_values.update({value.id: value for value in values})
+        return self.__class__(*existing_values.values())
+
+    def with_values_from(self: ValuesDictType, other: ValuesDict) -> ValuesDictType:
+        """Get a copy of the object, with overrides from another values dictionary.
+
+        Values from the other dictionary will override values from the
+        current dictionary.
+
+        :param other: the other dictionary to take values from.
+        :return: The new values dictionary.
+        """
+        return self.with_values(*other.values())
+
+    def without_values(self: ValuesDictType, *value_names: str) -> ValuesDictType:
+        """Get a copy of the object, without values with the given names.
+
+        This method will ignore value names that aren't present in the
+        original dictionary.
+
+        :param value_names: The name of the values to remove.
+        :return: The new values dictionary.
+        """
+        existing_values = {key: value for key, value in self.items()}
+        for value_name in value_names:
+            existing_values.pop(value_name, None)
+
+        return self.__class__(*existing_values.values())
+
 
 class Value:
     """
@@ -115,11 +154,14 @@ class Value:
         if v == '' and self.default != '' and (self.choices is None or v not in self.choices):
             raise ValueError('Value can\'t be empty')
         if self.regexp is not None and not re.match(self.regexp, str(v) if v is not None else ''):
-            raise ValueError('Value "%s" does not match regexp "%s"' % (self.show_value(v), self.regexp))
+            raise ValueError('Value does not match regexp "%s"' % self.regexp)
         if self.choices is not None and v not in self.choices:
             if not self.aliases or v not in self.aliases:
-                raise ValueError('Value "%s" is not in list: %s' % (
-                    self.show_value(v), ', '.join(str(s) for s in self.choices)))
+                raise ValueError(
+                    'Value is not in list: %s' % (
+                        ', '.join(str(s) for s in self.choices)
+                    )
+                )
 
     def load(self, domain, v, requests):
         """
@@ -250,7 +292,7 @@ class ValueFloat(Value):
         try:
             float(v)
         except ValueError:
-            raise ValueError('Value "%s" is not a float value' % self.show_value(v))
+            raise ValueError('Value is not a float value')
 
     def get(self):
         return float(self._value)
@@ -269,7 +311,7 @@ class ValueBool(Value):
                 'n', 'no',  '0', 'false', 'off',
             }:
 
-            raise ValueError('Value "%s" is not a boolean (y/n)' % self.show_value(v))
+            raise ValueError('Value is not a boolean (y/n)')
 
     def get(self):
         return (isinstance(self._value, bool) and self._value) or \
@@ -297,7 +339,7 @@ class ValueDate(Value):
                 continue
             return dateval
 
-        raise ValueError('Value "%s" does not match format in %s' % (self.show_value(v), self.accepted_formats))
+        raise ValueError('Value does not match format in %s' % self.accepted_formats)
 
     def check_valid(self, v):
         if self.required and not v:
@@ -313,7 +355,7 @@ class ValueDate(Value):
         if isinstance(v, datetime.date):
             self._value = v
         else:
-            raise ValueError('Value %r is not of the proper type' % self.show_value(v))
+            raise ValueError('Value is not of the proper type')
 
     def dump(self):
         if self._value:
